@@ -1,4 +1,4 @@
-// Copyright (C) 2004-2022 Artifex Software, Inc.
+// Copyright (C) 2004-2025 Artifex Software, Inc.
 //
 // This file is part of MuPDF.
 //
@@ -39,6 +39,7 @@ typedef struct fz_tuning_context fz_tuning_context;
 typedef struct fz_store fz_store;
 typedef struct fz_glyph_cache fz_glyph_cache;
 typedef struct fz_document_handler_context fz_document_handler_context;
+typedef struct fz_archive_handler_context fz_archive_handler_context;
 typedef struct fz_output fz_output;
 typedef struct fz_context fz_context;
 
@@ -137,6 +138,14 @@ const char *fz_caught_message(fz_context *ctx);
 */
 int fz_caught(fz_context *ctx);
 
+/*
+	Within an fz_catch() block, retrieve the errno code for
+	the current SYSTEM exception.
+
+	Is undefined for non-SYSTEM errors.
+*/
+int fz_caught_errno(fz_context *ctx);
+
 /**
 	Within an fz_catch() block, rethrow the current exception
 	if the errcode of the current exception matches.
@@ -144,6 +153,7 @@ int fz_caught(fz_context *ctx);
 	This assumes no intervening use of fz_try/fz_catch.
 */
 void fz_rethrow_if(fz_context *ctx, int errcode);
+void fz_rethrow_unless(fz_context *ctx, int errcode);
 
 /**
 	Format an error message, and log it to the registered
@@ -174,9 +184,11 @@ void fz_end_throw_on_repair(fz_context *ctx);
 #define fz_vwarn(CTX, FMT, VA) fz_vwarnFL(CTX, __FILE__, __LINE__, FMT, VA)
 #define fz_warn(CTX, ...) fz_warnFL(CTX, __FILE__, __LINE__, __VA_ARGS__)
 #define fz_rethrow_if(CTX, ERRCODE) fz_rethrow_ifFL(CTX, __FILE__, __LINE__, ERRCODE)
+#define fz_rethrow_unless(CTX, ERRCODE) fz_rethrow_unlessFL(CTX, __FILE__, __LINE__, ERRCODE)
 #define fz_log_error_printf(CTX, ...) fz_log_error_printfFL(CTX, __FILE__, __LINE__, __VA_ARGS__)
 #define fz_vlog_error_printf(CTX, FMT, VA) fz_log_error_printfFL(CTX, __FILE__, __LINE__, FMT, VA)
 #define fz_log_error(CTX, STR) fz_log_error_printfFL(CTX, __FILE__, __LINE__, STR)
+#define fz_do_catch(CTX) fz_do_catchFL(CTX, __FILE__, __LINE__)
 FZ_NORETURN void fz_vthrowFL(fz_context *ctx, const char *file, int line, int errcode, const char *fmt, va_list ap);
 FZ_NORETURN void fz_throwFL(fz_context *ctx, const char *file, int line, int errcode, const char *fmt, ...) FZ_PRINTFLIKE(5,6);
 FZ_NORETURN void fz_rethrowFL(fz_context *ctx, const char *file, int line);
@@ -184,22 +196,44 @@ void fz_morph_errorFL(fz_context *ctx, const char *file, int line, int fromcode,
 void fz_vwarnFL(fz_context *ctx, const char *file, int line, const char *fmt, va_list ap);
 void fz_warnFL(fz_context *ctx, const char *file, int line, const char *fmt, ...) FZ_PRINTFLIKE(4,5);
 void fz_rethrow_ifFL(fz_context *ctx, const char *file, int line, int errcode);
+void fz_rethrow_unlessFL(fz_context *ctx, const char *file, int line, int errcode);
 void fz_log_error_printfFL(fz_context *ctx, const char *file, int line, const char *fmt, ...) FZ_PRINTFLIKE(4,5);
 void fz_vlog_error_printfFL(fz_context *ctx, const char *file, int line, const char *fmt, va_list ap);
 void fz_log_errorFL(fz_context *ctx, const char *file, int line, const char *str);
+int fz_do_catchFL(fz_context *ctx, const char *file, int line);
 #endif
 
-enum
+/* Report an error to the registered error callback. */
+void fz_report_error(fz_context *ctx);
+
+/*
+ * Swallow an error and ignore it completely.
+ * This should only be called to signal that you've handled a TRYLATER or ABORT error,
+ */
+void fz_ignore_error(fz_context *ctx);
+
+/* Convert an error into another runtime exception.
+ * For use when converting an exception from Fitz to a language binding exception.
+ */
+const char *fz_convert_error(fz_context *ctx, int *code);
+
+enum fz_error_type
 {
-	FZ_ERROR_NONE = 0,
-	FZ_ERROR_MEMORY = 1,
-	FZ_ERROR_GENERIC = 2,
-	FZ_ERROR_SYNTAX = 3,
-	FZ_ERROR_MINOR = 4,
-	FZ_ERROR_TRYLATER = 5,
-	FZ_ERROR_ABORT = 6,
-	FZ_ERROR_REPAIRED = 7,
-	FZ_ERROR_COUNT
+	FZ_ERROR_NONE,
+	FZ_ERROR_GENERIC,
+
+	FZ_ERROR_SYSTEM, // fatal out of memory or syscall error
+	FZ_ERROR_LIBRARY, // unclassified error from third-party library
+	FZ_ERROR_ARGUMENT, // invalid or out-of-range arguments to functions
+	FZ_ERROR_LIMIT, // failed because of resource or other hard limits
+	FZ_ERROR_UNSUPPORTED, // tried to use an unsupported feature
+	FZ_ERROR_FORMAT, // syntax or format errors that are unrecoverable
+	FZ_ERROR_SYNTAX, // syntax errors that should be diagnosed and ignored
+
+	// for internal use only
+	FZ_ERROR_TRYLATER, // try-later progressive loading signal
+	FZ_ERROR_ABORT, // user requested abort signal
+	FZ_ERROR_REPAIRED, // internal flag used when repairing a PDF to avoid cycles
 };
 
 /**
@@ -555,6 +589,12 @@ const char *fz_user_css(fz_context *ctx);
 */
 void fz_set_user_css(fz_context *ctx, const char *text);
 
+/**
+	Set the user stylesheet by loading the source from a file.
+	If the file is missing, do nothing.
+*/
+void fz_load_user_css(fz_context *ctx, const char *filename);
+
 /**
 	Return whether to respect document styles in HTML and EPUB.
 */
@@ -662,6 +702,16 @@ void *fz_realloc(fz_context *ctx, void *p, size_t size);
 */
 void fz_free(fz_context *ctx, void *p);
 
+/**
+	Flexible array member allocation helpers.
+*/
+#define fz_malloc_flexible(ctx, T, M, count) \
+	Memento_label(fz_calloc(ctx, 1, offsetof(T, M) + sizeof(*((T*)0)->M) * (count)), #T)
+#define fz_realloc_flexible(ctx, p, T, M, count) \
+	Memento_label(fz_realloc(ctx, p, offsetof(T, M) + sizeof(*((T*)0)->M) * (count)), #T)
+#define fz_pool_alloc_flexible(ctx, pool, T, M, count) \
+	fz_pool_alloc(ctx, pool, offsetof(T, M) + sizeof(*((T*)0)->M) * (count))
+
 /**
 	fz_malloc equivalent that returns NULL rather than throwing
 	exceptions.
@@ -680,6 +730,17 @@ void *fz_calloc_no_throw(fz_context *ctx, size_t count, size_t size);
 */
 void *fz_realloc_no_throw(fz_context *ctx, void *p, size_t size);
 
+/**
+	fz_malloc equivalent, except that the block is guaranteed aligned.
+	Block must be freed later using fz_free_aligned.
+*/
+void *fz_malloc_aligned(fz_context *ctx, size_t size, int align);
+
+/**
+	fz_free equivalent, for blocks allocated via fz_malloc_aligned.
+*/
+void fz_free_aligned(fz_context *ctx, void *p);
+
 /**
 	Portable strdup implementation, using fz allocators.
 */
@@ -690,6 +751,34 @@ char *fz_strdup(fz_context *ctx, const char *s);
 */
 void fz_memrnd(fz_context *ctx, uint8_t *block, int len);
 
+/*
+	Reference counted malloced C strings.
+*/
+typedef struct
+{
+	int refs;
+	char str[FZ_FLEXIBLE_ARRAY];
+} fz_string;
+
+/*
+	Allocate a new string to hold a copy of str.
+
+	Returns with a refcount of 1.
+*/
+fz_string *fz_new_string(fz_context *ctx, const char *str);
+
+/*
+	Take another reference to a string.
+*/
+fz_string *fz_keep_string(fz_context *ctx, fz_string *str);
+
+/*
+	Drop a reference to a string, freeing if the refcount
+	reaches 0.
+*/
+void fz_drop_string(fz_context *ctx, fz_string *str);
+
+#define fz_cstring_from_string(A) ((A) == NULL ? NULL : (A)->str)
 
 /* Implementation details: subject to change. */
 
@@ -699,7 +788,7 @@ void fz_var_imp(void *);
 fz_jmp_buf *fz_push_try(fz_context *ctx);
 int fz_do_try(fz_context *ctx);
 int fz_do_always(fz_context *ctx);
-int fz_do_catch(fz_context *ctx);
+int (fz_do_catch)(fz_context *ctx);
 
 #ifndef FZ_JMPBUF_ALIGN
 #define FZ_JMPBUF_ALIGN 32
@@ -719,6 +808,7 @@ typedef struct
 	fz_error_stack_slot padding;
 	fz_error_stack_slot *stack_base;
 	int errcode;
+	int errnum; /* errno for SYSTEM class errors */
 	void *print_user;
 	void (*print)(void *user, const char *message);
 	char message[256];
@@ -742,13 +832,41 @@ typedef struct
 	float min_line_width;
 } fz_aa_context;
 
+typedef enum
+{
+	FZ_ACTIVITY_NEW_DOC = 0,
+	FZ_ACTIVITY_SHUTDOWN = 1
+} fz_activity_reason;
+
+typedef void (fz_activity_fn)(fz_context *ctx, void *opaque, fz_activity_reason reason, void *reason_arg);
+
+typedef struct
+{
+	void *opaque;
+	fz_activity_fn *activity;
+} fz_activity_context;
+
+void fz_register_activity_logger(fz_context *ctx, fz_activity_fn *activity, void *opaque);
+
 struct fz_context
 {
 	void *user;
+
+	/* If master points to itself, then we are the master context.
+	 * If master is NULL, then we are the master context, but we have
+	 * been destroyed. We exist just so the count of clones can live
+	 * on. Otherwise master points to the master context from which
+	 * we were cloned. */
+	fz_context *master;
+	/* The number of contexts in this family. 1 for this one, plus
+	 * 1 for every context cloned (directly or indirectly) from it. */
+	int context_count;
+
 	fz_alloc_context alloc;
 	fz_locks_context locks;
 	fz_error_context error;
 	fz_warn_context warn;
+	fz_activity_context activity;
 
 	/* unshared contexts */
 	fz_aa_context aa;
@@ -760,6 +878,7 @@ struct fz_context
 
 	/* TODO: should these be unshared? */
 	fz_document_handler_context *handler;
+	fz_archive_handler_context *archive;
 	fz_style_context *style;
 	fz_tuning_context *tuning;
 
